#!/usr/bin/perl -w
# read MNSDB Blacklists on stdin, filter and write black/whitelists
# to /etc/skolerouter.d/ProxyBlack||WhitelistSite + /etc/dansguardian/lists/exceptionsitelist
# 20130822: fix reckognition of negative cat ids for whitelist
my $usage = qq {blacklists.pl [-a]: 		hh/PL-RP, 12/2010
convert mnsdb black/whitelist into the req. squid/dansguardian files
-a: only append data from stdin to squid blacklist
};

my $tmp = "/tmp" ;
my $tsqbl = "$tmp/$$.sq-blacklist" ;
my $tsqwl = "$tmp/$$.sq-whitelist" ;
my $tdgwl = "$tmp/$$.dg-whitelist" ;

my $sqbl = "/etc/skolerouter.d/ProxyBlacklistSite" ;
my $sqwl = "/etc/skolerouter.d/ProxyWhitelistSite" ;
my $dgwl = "/etc/dansguardian/lists/exceptionsitelist" ;

my $verbose = shift || 0 ;
if($verbose =~ /-h/) {
  print $usage ;
  exit 0 ;
}

if($verbose eq "-a") { # append directly
  $verbose = shift || 0 ;  
  open(WH, ">>$sqbl") || die "can not append $sqbl\n" ;  
  my $c = 0 ;
  while(defined($line = <>)) {
    if($line =~ /^\*(.+)\s+(\d+)/) { # domain entry
      print WH ".$1\n" ;
      print "appending $1\n" if $verbose ;
      $c++ ;
    }
  }
  print "$c lines appended...\n" ;
  system("/etc/init.d/squid3 reload") ;
  exit ;
}



open(RH, "<$dgwl") || die ;
open(WH, ">$tdgwl") || die ;
open(WW, ">$tsqwl") || die ;
open(WB, ">$tsqbl") || die ;

while(<RH>) {
  last if /--MNS--AUTOGENERATED--TAIL/ ;
  print WH ;
}
close RH ;

print WH "#--MNS--AUTOGENERATED--TAIL\n" ;

# preparation done...
my %blackdom = () ;
my %blackurl = () ;
my %white = () ;
my $line ;

while(defined($line = <>)) {
  if($line !~ /^\#/) { # skip comments
	$line =~ s/\s+$//;
    if($line =~ /^\*(.+)\s+(\-?\d+)/) { # domain entry
      $blackdom{$1} = $2 ;
#	print "got black $1\n" if $verbose ;
    } elsif($line =~ /^--matchWL--\*?(.+)\s+(\-?\d+)/) {
      $white{$1} = $2 ;
	print "got white $1\n" if $verbose ;
    } elsif($line =~ /^([a-z0-9A-Z]\/.+)\s+(\-?\d+)/) { # url entry w/o proto
      $blackurl{$1} = $2 ;
	print "got blackURL $1\n" if $verbose ;
    } else {
#	print "unknown --$line--\n" if $verbose;
      # what???
    }
  }
}

sub isWhite {
  my $dom = shift ;
  while($dom =~ /\./) {
#    print "whitecheck $dom\n" ;
    return 1 if $white{$dom} ;
    $dom =~ s/^[^\.]*\.//;
  }
  # whole toplevel domains (like .com) should never be white- or blacklisted...
  return 0 ;
}

sub isImplied {
  my $dom = shift ;
  while($dom =~ s/^[^\.]+\.//) {
#    print "check implied $dom\n" ;
    return 1 if($blackdom{$dom});
  }
  return 0 ;
}

my $count = 0 ;
my $t = time ;

foreach (keys %blackdom) {
  print WB ".$_\n" unless(isWhite($_) || isImplied($_));
  if($verbose && time ne $t) {
    print chr(13) . "bd:" . $count++ . "   " ;
    $t = time ;
  }
}
print chr(13) . "b: " . $count++ . "   \n" if $verbose ;
close(WB) ;



$count = 0 ;
$t = time ;
foreach (keys %white) {
  print WW ".$_\n" ;
  if($verbose && time ne $t) {
    print chr(13) . "w: " . $count++ . "   " ;
    $t = time ;
  }
}
print chr(13) . "w: " . $count++ . "   \n" if $verbose ;

close WB ;
close WW ;
close WH ;



sub replace {
  my ($dest, $src) = (@_);
  rename($dest, $dest.".org") || return 0 ;
  open(IH, "<$src") || return 0 ;
  open(OH, ">$dest") || return 0 ;
  while(<IH>) { print OH ; }
  close IH ;
  close OH ;
  unlink($src) ;
  return 1 ;
}

replace($dgwl, $tdgwl) || die "can not replace $dgwl\n";
replace($sqbl, $tsqbl) || die "can not replace $sqbl\n";
replace($sqwl, $tsqwl) || die "can not replace $sqwl\n";
system("/etc/init.d/dansguardian reload") ;
system("/etc/init.d/squid3 reload") ;
